import codecs
import ipaddress
import math

import validators
from pyasn1_alt_modules import rfc5280, rfc8398

from pkilint import validation

_GENERALNAME_INSTANCE = rfc5280.GeneralName()
_GENERALNAME_TYPES = [str(n) for n in _GENERALNAME_INSTANCE.componentType]

OTHER_NAME_MAPPINGS = rfc5280.anotherNameMap.copy()


def validators_predicate(func, value):
    ret = func(value)

    return isinstance(ret, bool) and ret


def is_nameconstraints_child_node(node):
    return any((isinstance(n.pdu, rfc5280.NameConstraints) for n in node.parents))


def create_generalname_type_predicate(generalname_type):
    if generalname_type in _GENERALNAME_TYPES:
        return lambda n: (
                n.name == generalname_type and
                n.parent is not None and
                isinstance(n.parent.pdu, rfc5280.GeneralName)
        )
    else:
        raise ValueError(f'Invalid type specified: {generalname_type}')


class UriSyntaxValidator(validation.Validator):
    VALIDATION_INVALID_URI_SYNTAX = validation.ValidationFinding(
        validation.ValidationFindingSeverity.ERROR,
        'pkix.invalid_uri_syntax'
    )

    VALIDATION_LDAP_URI_NOT_VALIDATED = validation.ValidationFinding(
        validation.ValidationFindingSeverity.NOTICE,
        'pkix.ldap_uri_not_validated'
    )

    def __init__(self, **kwargs):
        super().__init__(validations=[self.VALIDATION_INVALID_URI_SYNTAX, self.VALIDATION_LDAP_URI_NOT_VALIDATED],
                         **kwargs
                         )

    def validate_value(self, node):
        value = str(node.pdu)

        if value.casefold().startswith('ldap://'.casefold()):
            raise validation.ValidationFindingEncountered(self.VALIDATION_LDAP_URI_NOT_VALIDATED, value)

        return validators_predicate(validators.url, value)

    def validate(self, node):
        if not self.validate_value(node):
            raise validation.ValidationFindingEncountered(
                self.VALIDATION_INVALID_URI_SYNTAX,
                f'Invalid URI syntax: "{str(node.pdu)}"'
            )


class DomainNameSyntaxValidator(validation.Validator):
    VALIDATION_NOT_PREFERRED_NAME_SYNTAX = validation.ValidationFinding(
        validation.ValidationFindingSeverity.ERROR,
        'pkix.invalid_domain_name_syntax'
    )

    def __init__(self, **kwargs):
        super().__init__(validations=[
            self.VALIDATION_NOT_PREFERRED_NAME_SYNTAX
        ],
            **kwargs
        )

    def validate_value(self, node):
        value = str(node.pdu)

        return validators_predicate(validators.domain, value)

    def validate(self, node):
        if not self.validate_value(node):
            raise validation.ValidationFindingEncountered(
                self.VALIDATION_NOT_PREFERRED_NAME_SYNTAX,
                f'Invalid domain name syntax: "{str(node.pdu)}"'
            )


class MailboxAddressSyntaxValidator(validation.Validator):
    VALIDATION_INVALID_EMAIL_ADDRESS_SYNTAX = validation.ValidationFinding(
        validation.ValidationFindingSeverity.ERROR,
        'pkix.invalid_email_address_syntax'
    )

    def __init__(self, **kwargs):
        super().__init__(validations=[
            self.VALIDATION_INVALID_EMAIL_ADDRESS_SYNTAX
        ],
            **kwargs
        )

    def validate_value(self, node):
        value = str(node.pdu)

        return validators_predicate(validators.email, value)

    def validate(self, node):
        if not self.validate_value(node):
            raise validation.ValidationFindingEncountered(
                self.VALIDATION_INVALID_EMAIL_ADDRESS_SYNTAX,
                f'Invalid e-mail address syntax: "{str(node.pdu)}"'
            )


class GeneralNameUriSyntaxValidator(UriSyntaxValidator):
    def __init__(self):
        super().__init__(
            predicate=create_generalname_type_predicate(
                'uniformResourceIdentifier'
            ),
        )

    def validate_value(self, node):
        is_nc_child = is_nameconstraints_child_node(node)

        value = str(node.pdu)

        if is_nc_child:
            if value == '.':
                return True

            if value.startswith('.'):
                value = value[1:]

            return validators_predicate(validators.domain, value)
        else:
            return super().validate_value(node)


class GeneralNameDnsNameSyntaxValidator(DomainNameSyntaxValidator):
    def __init__(self):
        super().__init__(
            predicate=create_generalname_type_predicate('dNSName')
        )

    def validate_value(self, node):
        value = str(node.pdu)

        if len(value) == 0 and is_nameconstraints_child_node(node):
            return True
        else:
            return super().validate_value(node)


class GeneralNameMailboxAddressSyntaxValidator(MailboxAddressSyntaxValidator):
    def __init__(self):
        super().__init__(
            predicate=create_generalname_type_predicate('rfc822Name'),
        )

    def validate_value(self, node):
        value = str(node.pdu)

        if '@' in value:
            return super().validate_value(node)

        if is_nameconstraints_child_node(node):
            if value == '.':
                return True

            if value.startswith('.'):
                value = value[1:]

            return validators_predicate(validators.domain, value)
        else:
            return super().validate_value(node)


def _get_cidr_prefix(address_octets):
    address_bitlen = len(address_octets) * 8

    address = int.from_bytes(address_octets, 'big')

    if address == 0:
        return 0

    lsb = int(math.log2(address & -address))

    for i in range(address_bitlen - 1, lsb, -1):
        if address & (1 << i) == 0:
            raise ValueError(f'Netmask bit index {i} is not asserted')

    return address_bitlen - lsb


class GeneralNameIpAddressSyntaxValidator(validation.Validator):
    VALIDATION_IP_ADDRESS_NC_WRONG_LENGTH = validation.ValidationFinding(
        validation.ValidationFindingSeverity.ERROR,
        'pkix.ip_address_name_constraint_wrong_length'
    )

    VALIDATION_IP_ADDRESS_WRONG_LENGTH = validation.ValidationFinding(
        validation.ValidationFindingSeverity.ERROR,
        'pkix.ip_address_wrong_length'
    )

    VALIDATION_IP_ADDRESS_CONSTRAINT_NOT_CIDR = validation.ValidationFinding(
        validation.ValidationFindingSeverity.ERROR,
        'pkix.ip_address_name_constraint_invalid_cidr'
    )

    def __init__(self):
        super().__init__(
            predicate=create_generalname_type_predicate('iPAddress'),
            validations=[
                self.VALIDATION_IP_ADDRESS_NC_WRONG_LENGTH,
                self.VALIDATION_IP_ADDRESS_WRONG_LENGTH,
                self.VALIDATION_IP_ADDRESS_CONSTRAINT_NOT_CIDR,
            ]
        )

    def validate(self, node):
        is_nc_child = is_nameconstraints_child_node(node)

        length_multiplier = 2 if is_nc_child else 1

        allowed_lengths = [address_lengths * length_multiplier for address_lengths in [4, 16]]

        value = node.pdu.asOctets()

        actual_length = len(value)

        if actual_length not in allowed_lengths:
            if is_nc_child:
                raise validation.ValidationFindingEncountered(
                    self.VALIDATION_IP_ADDRESS_NC_WRONG_LENGTH,
                    f'Invalid IP address contraint length: {actual_length}'
                )
            else:
                raise validation.ValidationFindingEncountered(
                    self.VALIDATION_IP_ADDRESS_WRONG_LENGTH,
                    f'Invalid IP address length: {actual_length}'
                )

        if is_nc_child:
            midpoint = int(actual_length / 2)

            network = value[0:midpoint]
            mask = value[midpoint:]

            try:
                cidr = _get_cidr_prefix(mask)

                if actual_length == (4 * 2):
                    network_obj = ipaddress.IPv4Address(network)

                    ipaddress.IPv4Network(f'{network_obj}/{cidr}')
                else:
                    network_obj = ipaddress.IPv6Address(network)

                    ipaddress.IPv6Network(f'{network_obj}/{cidr}')
            except ValueError as e:
                raise validation.ValidationFindingEncountered(
                    self.VALIDATION_IP_ADDRESS_CONSTRAINT_NOT_CIDR,
                    str(e)
                )


class SmtpUTF8MailboxValidator(validation.Validator):
    VALIDATION_NOT_MAILBOX_ADDRESS = validation.ValidationFinding(
        validation.ValidationFindingSeverity.ERROR,
        'pkix.smtp_utf8_mailbox_invalid_syntax'
    )

    VALIDATION_ASCII_ONLY = validation.ValidationFinding(
        validation.ValidationFindingSeverity.ERROR,
        'pkix.smtp_utf8_mailbox_is_ascii_only'
    )

    VALIDATION_HAS_BOM = validation.ValidationFinding(
        validation.ValidationFindingSeverity.ERROR,
        'pkix.smtp_utf8_mailbox_has_bom'
    )

    VALIDATION_DOMAIN_PART_NOT_LOWERCASE = validation.ValidationFinding(
        validation.ValidationFindingSeverity.ERROR,
        'pkix.smtp_utf8_mailbox_has_uppercase'
    )

    def __init__(self):
        super().__init__(
            validations=[
                self.VALIDATION_NOT_MAILBOX_ADDRESS,
                self.VALIDATION_ASCII_ONLY,
                self.VALIDATION_HAS_BOM,
                self.VALIDATION_DOMAIN_PART_NOT_LOWERCASE,
            ],
            pdu_class=rfc8398.SmtpUTF8Mailbox
        )

    def validate(self, node):
        value = str(node.pdu)

        if '@' not in value:
            raise validation.ValidationFindingEncountered(self.VALIDATION_NOT_MAILBOX_ADDRESS)

        local_part, domain_part = value.split('@', maxsplit=1)

        if local_part.isascii():
            raise validation.ValidationFindingEncountered(self.VALIDATION_ASCII_ONLY)

        encoded = local_part.encode('utf-8')
        if encoded.startswith(codecs.BOM_UTF8):
            raise validation.ValidationFindingEncountered(self.VALIDATION_HAS_BOM)

        if domain_part != domain_part.lower():
            raise validation.ValidationFindingEncountered(self.VALIDATION_DOMAIN_PART_NOT_LOWERCASE)



from pyasn1_alt_modules import rfc5280, rfc6962

from pkilint import common, validation
from pkilint.cabf.serverauth import serverauth_constants
from pkilint.pkix import Rfc2119Word

_CODE_CLASSIFIER = 'cabf.serverauth.cross_ca'


class CrossCertificateExtensionAllowanceValidator(common.ExtensionIdentifierAllowanceValidator):
    """Validates that the extensions conform with BR 7.1.2.2.3."""

    _EXTENSION_ALLOWANCES = {
        rfc5280.id_ce_authorityKeyIdentifier: Rfc2119Word.MUST,
        rfc5280.id_ce_basicConstraints: Rfc2119Word.MUST,
        rfc5280.id_ce_certificatePolicies: Rfc2119Word.MUST,
        rfc5280.id_ce_cRLDistributionPoints: Rfc2119Word.MUST,
        rfc5280.id_ce_keyUsage: Rfc2119Word.MUST,
        rfc5280.id_ce_subjectKeyIdentifier: Rfc2119Word.MUST,
        rfc5280.id_pe_authorityInfoAccess: Rfc2119Word.SHOULD,
        rfc5280.id_ce_nameConstraints: Rfc2119Word.MUST_NOT,
        rfc6962.id_ce_embeddedSCT: Rfc2119Word.MAY
    }

    def __init__(self, certificate_type):
        self._extension_allowances = self._EXTENSION_ALLOWANCES.copy()

        if certificate_type == serverauth_constants.CertificateType.EXTERNAL_CROSS_CA:
            eku_allowance_word = Rfc2119Word.MUST
        elif certificate_type == serverauth_constants.CertificateType.INTERNAL_CROSS_CA:
            eku_allowance_word = Rfc2119Word.SHOULD
        else:
            raise ValueError(f'Unsupported certificate type: {certificate_type}')

        self._extension_allowances[rfc5280.id_ce_extKeyUsage] = eku_allowance_word

        super().__init__(self._extension_allowances, _CODE_CLASSIFIER, Rfc2119Word.SHOULD_NOT)


class CrossCertificateAllowedEkuValidator(common.ExtendedKeyUsageAllowanceValidator):
    """Validates that the content of the extended key usage conforms to BR 7.1.2.2.4 and 7.1.2.2.5."""

    _RESTRICTED_EKU_ALLOWANCES = {
        rfc5280.id_kp_serverAuth: Rfc2119Word.MUST,
        rfc5280.id_kp_clientAuth: Rfc2119Word.MAY,
        rfc5280.id_kp_emailProtection: Rfc2119Word.MUST_NOT,
        rfc5280.id_kp_codeSigning: Rfc2119Word.MUST_NOT,
        rfc5280.id_kp_timeStamping: Rfc2119Word.MUST_NOT,
        rfc5280.id_kp_OCSPSigning: Rfc2119Word.MUST_NOT,
    }

    VALIDATION_EXTERNAL_CROSS_CA_ANYEKU_PRESENT = validation.ValidationFinding(
        validation.ValidationFindingSeverity.ERROR,
        _CODE_CLASSIFIER + '.external_anyeku_present'
    )

    VALIDATION_INTERNAL_CROSS_CA_ANYEKU_WITH_OTHER_EKU = validation.ValidationFinding(
        validation.ValidationFindingSeverity.ERROR,
        _CODE_CLASSIFIER + '.internal_with_anyeku_and_other_eku'
    )

    def __init__(self, certificate_type: serverauth_constants.CertificateType):
        self._certificate_type = certificate_type

        super().__init__(self._RESTRICTED_EKU_ALLOWANCES, _CODE_CLASSIFIER, Rfc2119Word.SHOULD_NOT)

        self._validations.extend([
            self.VALIDATION_INTERNAL_CROSS_CA_ANYEKU_WITH_OTHER_EKU,
            self.VALIDATION_EXTERNAL_CROSS_CA_ANYEKU_PRESENT
        ])

    def validate(self, node):
        ekus = {n.pdu for n in node.children.values()}

        if rfc5280.anyExtendedKeyUsage in ekus:
            if self._certificate_type == serverauth_constants.CertificateType.EXTERNAL_CROSS_CA:
                raise validation.ValidationFindingEncountered(self.VALIDATION_EXTERNAL_CROSS_CA_ANYEKU_PRESENT)
            elif self._certificate_type == serverauth_constants.CertificateType.INTERNAL_CROSS_CA:
                if len(node.children) != 1:
                    raise validation.ValidationFindingEncountered(
                        self.VALIDATION_INTERNAL_CROSS_CA_ANYEKU_WITH_OTHER_EKU
                    )
            else:
                raise ValueError(f'Unsupported certificate type: {self._certificate_type}')
        else:
            return super().validate(node)



from pyasn1_alt_modules import rfc5280

from pkilint import validation


class VersionPresenceValidator(validation.NodePresenceValidator):
    VALIDATION_VERSION_MISSING = validation.ValidationFinding(
        validation.ValidationFindingSeverity.ERROR,
        'pkix.crl_version_missing'
    )

    def __init__(self):
        node_retriever = lambda n: n.navigate('version')

        super().__init__(node_retriever=node_retriever,
                         absence_finding=self.VALIDATION_VERSION_MISSING,
                         pdu_class=rfc5280.TBSCertList
                         )


class CorrectVersionValidator(validation.ScalarFieldValueEqualityValidator):
    def __init__(self):
        super().__init__(
            path='certificateList.tbsCertList.version',
            value=rfc5280.Version.namedValues['v2'],
            validations=validation.ValidationFinding(
                validation.ValidationFindingSeverity.ERROR,
                'pkix.crl_version_is_not_v2'
            )
        )


class SignatureAlgorithmMatchValidator(validation.DEREqualityValidator):
    def __init__(self):
        super().__init__(
            other_node_retriever=(
                lambda n: n.navigate('^.tbsCertList.signature')
            ),
            path='signatureAlgorithm',
            validation=validation.ValidationFinding(
                validation.ValidationFindingSeverity.ERROR,
                'pkix.crl_signature_algorithm_match'
            )
        )



#!/usr/bin/env python

import argparse
import sys

from pkilint import loader
from pkilint import report, util
from pkilint.pkix import certificate, name, extension


def main(cli_args=None) -> int:
    parser = argparse.ArgumentParser(description='RFC 5280 Certificate Linter')

    subparsers = parser.add_subparsers(dest='command', required=True)
    subparsers.add_parser('validations', help='Output the set of validations which this linter performs')

    lint_parser = subparsers.add_parser('lint', help='Lint the specified certificate')
    util.add_standard_args(lint_parser)

    lint_parser.add_argument('file', type=argparse.FileType('rb'),
                             help='The certificate to lint'
                             )

    args = parser.parse_args(cli_args)

    doc_validator = certificate.create_pkix_certificate_validator_container(
        certificate.create_decoding_validators(name.ATTRIBUTE_TYPE_MAPPINGS, extension.EXTENSION_MAPPINGS),
        [
            certificate.create_issuer_validator_container(
                []
            ),
            certificate.create_validity_validator_container(),
            certificate.create_subject_validator_container(
                []
            ),
            certificate.create_extensions_validator_container(
                []
            ),
        ]
    )

    if args.command == 'validations':
        print(report.report_included_validations(doc_validator))

        return 0
    else:
        cert = loader.load_certificate(args.file, args.file.name)

        results = doc_validator.validate(cert.root)

        print(args.format(results, args.severity))

        return report.get_findings_count(results, args.severity)


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

import argparse
import sys

from pkilint import loader, pkix, report, util
from pkilint.cabf import cabf_crl
from pkilint.pkix import crl, name, extension


def _add_args(parser):
    parser.add_argument('-t', '--type',
                        required=True,
                        type=str.upper,
                        choices=['CRL', 'ARL'],
                        help='The type of CRL (whether the CRL in question is a CRL or an ARL)'
                        )
    parser.add_argument('-p', '--profile',
                        required=True,
                        type=str.upper,
                        choices=['PKIX', 'BR'],
                        default='PKIX',
                        help='The profile against which to lint'
                        )


def main(cli_args=None) -> int:
    parser = argparse.ArgumentParser(description='RFC 5280 and CA/B Forum CRL Linter')

    subparsers = parser.add_subparsers(dest='command', required=True)
    validations_parser = subparsers.add_parser('validations',
                                               help='Output the set of validations which this linter performs')
    _add_args(validations_parser)

    lint_parser = subparsers.add_parser('lint', help='Lint the specified CRL')
    _add_args(lint_parser)
    util.add_standard_args(lint_parser)

    lint_parser.add_argument('file', type=argparse.FileType('rb'),
                             help='The CRL file to lint'
                             )

    args = parser.parse_args(cli_args)

    crl_type = crl.CertificateRevocationListType[args.type]

    validity_additional_validators = []
    doc_additional_validators = []

    if args.profile == 'BR':
        doc_additional_validators.append(
            cabf_crl.create_reason_code_validator(crl_type)
        )

        validity_additional_validators.append(
            cabf_crl.create_validity_period_validator(crl_type)
        )

    doc_validator = crl.create_pkix_crl_validator_container(
        [
            pkix.create_attribute_decoder(name.ATTRIBUTE_TYPE_MAPPINGS),
            pkix.create_extension_decoder(extension.EXTENSION_MAPPINGS),
        ],
        [
            crl.create_issuer_validator_container(
                []
            ),
            crl.create_validity_validator_container(
                validity_additional_validators
            ),
            crl.create_extensions_validator_container(
                []
            ),
        ] + doc_additional_validators
    )

    if args.command == 'validations':
        print(report.report_included_validations(doc_validator))

        return 0
    else:
        crl_doc = loader.load_crl(args.file, args.file.name)

        results = doc_validator.validate(crl_doc.root)

        print(args.format(results, args.severity))

        return report.get_findings_count(results, args.severity)


if __name__ == "__main__":
    sys.exit(main())



import csv
import io
import subprocess

from pkilint.cabf.serverauth import serverauth_constants
from pkilint.cabf.smime import smime_constants


def _test_program(name, args=None):
    if args is None:
        args = []

    output = subprocess.check_output([name, 'validations'] + args).decode()

    s = io.StringIO(output)

    c = csv.DictReader(s)
    row_count = len([r for r in c])

    assert row_count > 0


def test_lint_cabf_serverauth_cert():
    for cert_type in serverauth_constants.CertificateType:
        _test_program('lint_cabf_serverauth_cert', ['-t', cert_type.name.replace('_', '-')])


def test_lint_cabf_smime_cert():
    for g in smime_constants.Generation:
        for v in smime_constants.ValidationLevel:
            _test_program('lint_cabf_smime_cert', ['-t', f'{v}-{g}'])


def test_lint_crl():
    for p in ['BR', 'PKIX']:
        for t in ['CRL', 'ARL']:
            _test_program('lint_crl', ['-p', p, '-t', t])


def test_lint_ocsp_response():
    _test_program('lint_ocsp_response')


def test_lint_pkix_cert():
    _test_program('lint_pkix_cert')


def test_lint_pkix_signer_signee_cert_chain():
    _test_program('lint_pkix_signer_signee_cert_chain')



import functools
import glob
import sys
from os import path

from pkilint.cabf import smime
from pkilint.cabf.smime import smime_constants
from pkilint.pkix import certificate
from tests import integration_certificate

this_module = sys.modules[__name__]


for validation_level in smime_constants.ValidationLevel:
    for generation in smime_constants.Generation:
        cur_dir = path.dirname(__file__)

        test_dir = path.join(cur_dir, 'smime_br', validation_level.name.lower(), generation.name.lower())

        files = glob.glob(path.join(test_dir, '*.crttest'))

        for file in files:
            validator = certificate.create_pkix_certificate_validator_container(
                smime.create_decoding_validators(),
                smime.create_subscriber_validators(
                    validation_level, generation
                )
            )

            file_no_ext, _ = path.splitext(path.basename(file))

            func_name = f'test_{validation_level}-{generation}_{file_no_ext}'

            setattr(this_module, func_name, functools.partial(integration_certificate.run_test, file, validator))



import enum
from typing import Optional

from pyasn1.type.constraint import ValueRangeConstraint
from pyasn1_alt_modules import rfc5280

from pkilint import validation
from pkilint.document import ValueDecoder
from pkilint.pkix import extension, algorithm, name


def create_attribute_decoder(type_mappings, decode_unknown_as_directorystring=True):
    default = rfc5280.DirectoryString() if decode_unknown_as_directorystring else None

    decoder = ValueDecoder(
        type_path='type',
        value_path='value',
        type_mappings=type_mappings,
        default=default
    )

    return name.NameDecodingValidator(decode_func=decoder)


def create_extension_decoder(type_mappings):
    decoder = ValueDecoder(
        type_path='extnID',
        value_path='extnValue',
        type_mappings=type_mappings
    )

    return extension.ExtensionsDecodingValidator(decode_func=decoder)


def create_signature_algorithm_identifier_decoder(type_mappings, **kwargs):
    decoder = ValueDecoder(
        type_path='algorithm',
        value_path='parameters',
        type_mappings=type_mappings
    )

    return algorithm.AlgorithmIdentifierDecodingValidator(
        decode_func=decoder,
        **kwargs
    )


def create_name_validator_container(additional_validators=None, **kwargs):
    if additional_validators is None:
        additional_validators = []
    return validation.ValidatorContainer(
        validators=[
                       name.RDNContainsUniqueTypesValidator(),
                   ] + additional_validators,
        **kwargs
    )


class CertificateSerialNumberValidator(validation.ASN1ConstraintValidator):
    VALIDATION_FINDING_CERTIFICATE_SERIAL_NUMBER_OOR = validation.ValidationFinding(
        validation.ValidationFindingSeverity.ERROR,
        'pkix.certificate_serial_number_out_of_range'
    )

    MAX_VALUE = (1 << 159) - 1

    def __init__(self):
        super().__init__(
            pdu_class=rfc5280.CertificateSerialNumber,
            validations=[
                self.VALIDATION_FINDING_CERTIFICATE_SERIAL_NUMBER_OOR
            ],
            constraint=ValueRangeConstraint(1, self.MAX_VALUE)
        )


class Rfc2119Word(enum.IntEnum):
    SHALL = 1
    MUST = SHALL
    SHOULD = 2
    MAY = 3
    MUST_NOT = 4
    SHALL_NOT = MUST_NOT
    SHOULD_NOT = 5

    def __str__(self):
        return self.name

    @property
    def to_severity(self) -> Optional[validation.ValidationFindingSeverity]:
        if self.value in {Rfc2119Word.SHALL, Rfc2119Word.SHALL_NOT}:
            return validation.ValidationFindingSeverity.ERROR
        elif self.value in {Rfc2119Word.SHOULD, Rfc2119Word.SHOULD_NOT}:
            return validation.ValidationFindingSeverity.WARNING

        return None



from pyasn1.type import constraint
from pyasn1.type.char import PrintableString

from pkilint import validation


def _char_range(start, end):
    return [chr(i) for i in range(ord(start), ord(end) + 1)]


class PrintableStringConstraintValidator(validation.ASN1ConstraintValidator):

    def __init__(self):
        allowed_chars = (
                _char_range('0', '9') +
                _char_range('A', 'Z') +
                _char_range('a', 'z') +
                list(" '()+,-./:=?")
        )
        c = constraint.PermittedAlphabetConstraint(*allowed_chars)

        super().__init__(pdu_supertype=PrintableString(), constraint=c,
                         validations=validation.ValidationFinding(
                             validation.ValidationFindingSeverity.ERROR,
                             'itu.invalid_printablestring_character'
                         )
                         )



import binascii

from pyasn1.codec.der.encoder import encode
from pyasn1.type import univ
from pyasn1_alt_modules import rfc4055, rfc5280, rfc5480, rfc8410

from pkilint import validation, document

SIGNATURE_ALGORITHM_IDENTIFIER_MAPPINGS = {
    **{o: document.ValueDecoder.VALUE_NODE_ABSENT for o in (
        rfc8410.id_Ed448,
        rfc8410.id_Ed25519,
        rfc8410.id_X448,
        rfc8410.id_X25519,
        rfc5480.id_dsa_with_sha1,
        rfc5480.id_dsa_with_sha224,
        rfc5480.id_dsa_with_sha256,
        rfc5480.ecdsa_with_SHA1,
        rfc5480.ecdsa_with_SHA224,
        rfc5480.ecdsa_with_SHA256,
        rfc5480.ecdsa_with_SHA384,
        rfc5480.ecdsa_with_SHA512,
    )},
    **{o: univ.Null() for o in (
        rfc5480.md2WithRSAEncryption,
        rfc5480.md5WithRSAEncryption,
        rfc5480.sha1WithRSAEncryption,
        rfc4055.sha224WithRSAEncryption,
        rfc4055.sha256WithRSAEncryption,
        rfc4055.sha384WithRSAEncryption,
        rfc4055.sha512WithRSAEncryption,
    )},
    rfc4055.id_RSASSA_PSS: rfc4055.RSASSA_PSS_params(),
}


class AlgorithmIdentifierDecodingValidator(validation.DecodingValidator):
    def __init__(self, *, decode_func, **kwargs):
        super().__init__(pdu_class=rfc5280.AlgorithmIdentifier,
                         decode_func=decode_func,
                         **kwargs
                         )


class AllowedSignatureAlgorithmEncodingValidator(validation.Validator):
    def __init__(self, *, validation, allowed_encodings, **kwargs):
        self._allowed_encodings = allowed_encodings

        super().__init__(
            validations=[validation],
            **kwargs
        )

    def validate(self, node):
        encoded = encode(node.pdu)

        if encoded not in self._allowed_encodings:
            encoded_str = binascii.hexlify(encoded).decode('us-ascii')

            raise validation.ValidationFindingEncountered(
                self._validations[0],
                f'Prohibited encoding: {encoded_str}'
            )



import operator
from datetime import timedelta

from pyasn1_alt_modules import rfc5280

from pkilint import validation
from pkilint.pkix import crl, time
from pkilint.pkix.crl import crl_extension


def create_validity_period_validator(
        crl_type: crl.CertificateRevocationListType
):
    if crl_type == crl.CertificateRevocationListType.CRL:
        max_validity_days = 10
        finding = 'cabf.crl_invalid_validity_period'
    else:
        max_validity_days = 365  # TODO: handle leap years?
        finding = 'cabf.arl_invalid_validity_period'

    thresholds = [
        (
            operator.le,
            timedelta(days=max_validity_days),
            validation.ValidationFinding(
                validation.ValidationFindingSeverity.ERROR,
                finding
            )
        )
    ]

    return time.ValidityPeriodThresholdsValidator(
        path='certificateList.tbsCertList.thisUpdate',
        end_validity_node_retriever=lambda n: n.navigate('^.nextUpdate'),
        validity_period_thresholds=thresholds
    )


def create_reason_code_validator(
        crl_type: crl.CertificateRevocationListType
):
    allowed_reasons = [
        rfc5280.CRLReason.namedValues[r]
        for r in [
            'keyCompromise',
            'affiliationChanged',
            'superseded',
            'cessationOfOperation',
            'privilegeWithdrawn',
        ]
    ]

    if crl_type == crl.CertificateRevocationListType.ARL:
        allowed_reasons.append(rfc5280.CRLReason.namedValues['cACompromise'])

    return crl_extension.CrlReasonCodeAllowlistValidator(
        allowed_reason_codes=allowed_reasons
    )



from pyasn1_alt_modules import rfc3739, rfc2985

from pkilint.cabf.asn1 import ev_guidelines as ev_guidelines_asn1
from pkilint.itu import x520_name
from pkilint.pkix import extension, name

NAME_ATTRIBUTE_MAPPINGS = {
    **rfc2985._certificateAttributesMapUpdate,
    **x520_name.ATTRIBUTE_TYPE_MAPPINGS,
    **name.ATTRIBUTE_TYPE_MAPPINGS,
    **ev_guidelines_asn1.ATTRIBUTE_TYPE_MAPPINGS,
}

EXTENSION_MAPPINGS = {
    **extension.EXTENSION_MAPPINGS,
    **ev_guidelines_asn1.EXTENSION_MAPPINGS,
    **rfc3739.certificateExtensionsMap,
}
